#include <font610.h>
#include <miac.h>

// This demo sketch can be uploaded to two MIAC devices connect via CAN
// They will send key press information to each other.

// Initialise the MIAC and CAN for 500kbps
miac miac(500);

uint8_t keyPressed;
uint8_t keyReceived;

void setup() {
  // Print intial text to the screen
  miac.print("CAN Example\npress any key\n\npressed\nreceived");
  // Setup the CAN message Identifier
  // for buffer 0, use Extended ID, ID = 200
  miac.txIdentWrite(0, 1, 200);
}

void loop() {
  // Test for a key pressed
  keyPressed = miac.keypadRead();
  if (keyPressed != 255)
  {

    // Set the text cursor to column 9, row 3
    miac.cursor(9,3);
    // Print the key number to screen
    miac.print(keyPressed);
    // Write the key data into CAN buffer 0, 1 byte to send
    miac.txDataWrite(0, 1, &keyPressed);
    // Invoke the sending of CAN buffer 0
    miac.txSend(0);
    // Wait for keypad to be released
    while (miac.keypadRead() != 255) delay(100);
    // Clear the displayed key
    miac.cursor(9,3);
    miac.print(' ');    
  }

  // Test for a received CAN message
  if (miac.rxCheck(0))
  {
    // Test that we have at least one data byte in the message
    if (miac.rxDataCountRead(0) > 0)
    {
      // A message of at least one byte received
      // Get the data from index 0 of the CAN receiver buffer 0
      keyReceived = miac.rxDataRead(0, 0); 
      // Print it to the screen
      miac.cursor(9,4);
      if (keyReceived == 255)
      {
        miac.print(' ');
      }
      else
      {
        miac.print(keyReceived);
      }
    }
  }
}


