#include <font610.h>
#include <miac.h>

miac miac;

char theDate[8];
char theTime[8];
char onTime[8];
char previousMins = 0;

uint8_t heaterDuration = 0;    // length of time heater should be on, in minutes
uint8_t heaterTimer = 0; // count down timer for heater on time, in minutes

uint8_t menuSelection = 0;
uint8_t menuActive = 0;

enum keyType {KEY_F2, KEY_LEFT, KEY_F1, KEY_DOWN, KEY_OK, KEY_UP, KEY_RETURN, KEY_RIGHT, KEY_MENU, KEY_NONE=255} key = KEY_NONE;

enum triggerType {TRIGGER_NONE, TRIGGER_RTC, TRIGGER_MANUAL} triggerSource = TRIGGER_NONE;

struct icon
{
  uint8_t width;
  uint8_t height;
  uint8_t data[32];
};

icon iconHeater = {16,16,{0x6d,0xb6,0x6d,0xb6,0x6d,0xb6,0x6d,0xb6,0x92,0x49,0xff,0xff,0xff,0xff,0xe6,0x67,0xf7,0x77,0xf3,0x33,0xf7,0x77,0xe6,0x67,0xee,0xef,0xcc,0xcf,0xee,0xef,0xe6,0x67}};
icon iconClock = {16,16,{0xff,0xff,0xfc,0x1f,0xf3,0xe7,0xef,0xfb,0xdf,0xfd,0xdf,0xfd,0xbf,0xfe,0xbf,0x7e,0xbe,0xbe,0xbd,0xde,0xdb,0xfd,0xd7,0xfd,0xef,0xfb,0xf3,0xe7,0xfc,0x1f,0xff,0xff}};
icon iconTimer = {16,16,{0xff,0xff,0xfc,0x1f,0xf3,0xe7,0xef,0xfb,0xdf,0xfd,0xdf,0xfd,0xbf,0xfe,0xbf,0x7e,0xbf,0x3e,0xbf,0x1e,0xdf,0x0d,0xdf,0x05,0xef,0x03,0xf3,0x07,0xfc,0x1f,0xff,0xff}};


void drawIcon(icon *g, uint8_t x, uint8_t y)
{
  uint8_t w;
  uint8_t h;
  uint16_t p = 0;
  for (h=0; h < g->height; ++h)
  {
    for (w=0; w < g->width; ++w)
    {
      miac.pixel(x+w, y + g->height - h -1, ((g->data[p/8])&(1<<7-(p%8)))?0:1  );
      ++p;
    }
  }
}

void clearArea(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t state)
{
  uint8_t vw;
  uint8_t vh;
  for (vh=0; vh < h; ++vh)
  {
    for (vw=0; vw < w; ++vw)
    {
      miac.pixel(x+vw, y+vh, state);
    }
  }
}

#define HEATER_X  5
#define HEATER_Y  33
#define CLOCK_X   47
#define CLOCK_Y   33
#define TIMER_X   26
#define TIMER_Y   33

void drawHeater(void)
{
  drawIcon(&iconHeater, HEATER_X, HEATER_Y);
}

void clearHeater(void)
{
  clearArea(HEATER_X,HEATER_Y,16,16,0);
}

void drawClock(void)
{
  drawIcon(&iconClock, CLOCK_X, CLOCK_Y);
}

void clearClock(void)
{
  clearArea(CLOCK_X,CLOCK_Y,16,16,0);
}

void drawTimer(void)
{
  drawIcon(&iconTimer, TIMER_X, TIMER_Y);
}

void clearTimer(void)
{
  clearArea(TIMER_X,TIMER_Y,16,16,0);
}

void showIcons(void)
{
  if (heaterTimer)
  {
    drawHeater();
    switch(triggerSource)
    {
      case TRIGGER_RTC: drawClock();
      case TRIGGER_MANUAL: drawTimer();
    }
  }
  else
  {
    clearHeater();
    clearTimer();
    clearClock();
  }
}

void menuOn(void)
{
  menuActive = 1;
  menuSelection = 0;
  miac.clear();
  miac.cursor(0,1); miac.print("Time ");
  miac.cursor(0,3); miac.print("On       for     min");
  printOnTime();
  printDuration();
  drawCursor();
}

void menuOff(void)
{
  menuActive = 0;
  miac.clear();
  showIcons();
  printTimer();
  printTime();
}

void moveCursor(void)
{
  switch (menuSelection)
  {
    // clock time setting
    case 0: miac.cursor(7,2); break;
    case 1: miac.cursor(8,2); break;
    case 2: miac.cursor(10,2); break;
    case 3: miac.cursor(11,2); break;

    case 4: miac.cursor(3,4); break;
    case 5: miac.cursor(4,4); break;
    case 6: miac.cursor(6,4); break;
    case 7: miac.cursor(7,4); break;

    case 8: miac.cursor(13,4); break;
    case 9: miac.cursor(14,4); break;
    case 10: miac.cursor(15,4); break;
  }
}

void drawCursor(void)
{
  moveCursor();
  miac.print('^');
}

void clearCursor(void)
{
  moveCursor();
  miac.print(' ');
}

void left()
{
  clearCursor();
  if (menuSelection > 0)
  {
    --menuSelection;
  }
  else
  {
    menuSelection = 10;
  }
  drawCursor();
}

void right()
{
  clearCursor();
  if (menuSelection < 10)
  {
    ++menuSelection;
  }
  else
  {
    menuSelection = 0;
  }
  drawCursor();
}

void incTime(uint8_t i, uint8_t max)
{
  if (theTime[i] < (max + '0'))
  {
    theTime[i] += 1;
    theTime[4] = '0'; // zero the seconds
    theTime[5] = '0';
    miac.timeWrite(theTime);
  }
}

void decTime(uint8_t i)
{
  if (theTime[i] > '0')
  {
    theTime[i] -= 1;
    theTime[4] = '0'; // zero the seconds
    theTime[5] = '0';
    miac.timeWrite(theTime);
  }
}

void incOnTime(uint8_t i, uint8_t max)
{
  if (onTime[i] < (max + '0'))
  {
    onTime[i] += 1;
    printOnTime();
    saveOnTime();
  }
}

void decOnTime(uint8_t i)
{
  if (onTime[i] > '0')
  {
    onTime[i] -= 1;
    printOnTime();
    saveOnTime();
  }
}

void incDuration(uint8_t d)
{
  if (heaterDuration + d <= 240)
  {
    heaterDuration += d;
    printDuration();
    saveDuration();
  }
}

void decDuration(uint8_t d)
{
  if (heaterDuration > d)
  {
    heaterDuration -= d;
    printDuration();
    saveDuration();
  }
}

// Save the duration setting to NVM
void saveDuration(void)
{
  miac.eepromWrite(105, heaterDuration);
}

// Retrieve the duration setting from NVM
void getDuration(void)
{
  heaterDuration = miac.eepromRead(105);
  if (heaterDuration > 240)
  {
    heaterDuration = 240;
    saveDuration();
  }
}

void saveOnTime(void)
{
  miac.eepromWrite(100, onTime[0]);
  miac.eepromWrite(101, onTime[1]);
  miac.eepromWrite(102, onTime[2]);
  miac.eepromWrite(103, onTime[3]);
}

void getOnTime(void)
{
  uint8_t i;
  uint8_t error = 0;
  for (i=0; i<4; ++i)
  {
    onTime[i] = miac.eepromRead(100+i);
    if ((onTime[i] < '0')||(onTime[i] > '9'))
    {
      error = 1;
    }
  }
  if (error)
  {
    onTime[0] = '1';
    onTime[1] = '2';
    onTime[2] = '0';
    onTime[3] = '0';
    saveOnTime();
  }
}

  
void up()
{
  switch (menuSelection)
  {
    case 0: incTime(0,2); break;
    case 1: incTime(1,9); break;
    case 2: incTime(2,5); break;
    case 3: incTime(3,9); break;
    
    case 4: incOnTime(0,2); break;
    case 5: incOnTime(1,9); break;
    case 6: incOnTime(2,5); break;
    case 7: incOnTime(3,9); break;

    case 8: incDuration(100); break;
    case 9: incDuration(10); break;
    case 10: incDuration(1); break;
  }
}

void down()
{
  switch (menuSelection)
  {
    case 0: decTime(0); break;
    case 1: decTime(1); break;
    case 2: decTime(2); break;
    case 3: decTime(3); break;
    
    case 4: decOnTime(0); break;
    case 5: decOnTime(1); break;
    case 6: decOnTime(2); break;
    case 7: decOnTime(3); break;

    case 8: decDuration(100); break;
    case 9: decDuration(10); break;
    case 10: decDuration(1); break;
  }
}

void printTime(void)
{
  miac.timeRead(theTime);
  miac.cursor(7,1);
  // print the time HH:MM
  miac.print(theTime[0]);
  miac.print(theTime[1]);
  miac.print(':');
  miac.print(theTime[2]);
  miac.print(theTime[3]);
}


void printOnTime(void)
{
  miac.cursor(3,3);
  // print the time HH:MM
  miac.print(onTime[0]);
  miac.print(onTime[1]);
  miac.print(':');
  miac.print(onTime[2]);
  miac.print(onTime[3]);
}

void printDuration(void)
{
  char text[4];
  uint8_t i = 0;
  text[0] = (heaterDuration/100) + '0';
  text[1] = ((heaterDuration%100)/10) + '0';
  text[2] = (heaterDuration%10) + '0';
  text[3] = 0;
  while ((text[i] == '0')&&(i<2)) text[i++] = ' ';
  miac.cursor(13,3);
  miac.print(text);
}

void heaterOn(void)
{
  miac.relayOn(1);
  showIcons();
  printTimer();
}

void heaterOff(void)
{
  miac.relayOff(1);
  showIcons();
  printTimer();
}

void heaterStart(triggerType source)
{
  triggerSource = source;
  heaterTimer = heaterDuration;
  heaterOn();
}

void heaterStop(void)
{
  heaterTimer = 0;
  heaterOff();
}

void processTimer(void)
{
  if (previousMins != theTime[3])
  {
    previousMins = theTime[3];
    if (heaterTimer)
    {
      if (--heaterTimer == 0)
      {
        heaterOff();
      }
    printTimer();
    }
  }
}

void printTimer(void)
{
  if (!menuActive)
  {
    miac.cursor(1,2);
    if (heaterTimer)
    {
      miac.print(heaterTimer); miac.print(" minutes to off  ");
    }
    else
    {
      miac.print("                   ");
    }
  }
}

void setup()
{
  // get stored parameters
  getDuration();
  getOnTime();
  menuOff();
}

void loop()
{
  delay(20);
  printTime();
  processTimer();

  // If input I1 is active then start the heater in manual mode
  if (miac.digitalRead(1))
  {
    heaterStart(TRIGGER_MANUAL);
  }
  
  key = (keyType)miac.keypadRead();
  if (menuActive)
  {
    switch (key)
    {
      case KEY_LEFT: left(); break;
      case KEY_RIGHT: right(); break;
      case KEY_DOWN: down(); break;
      case KEY_UP: up(); break;
      case KEY_OK: menuOff(); break;
      case KEY_RETURN: menuOff(); break;
    }
  }
  else
  {
    switch (key)
    {
      case KEY_OK: heaterStart(TRIGGER_MANUAL); break;
      case KEY_RETURN: heaterStop(); break;
      case KEY_MENU: menuOn(); break;
    }
  }
}

